import '../../animated_infinite_scroll_pagination.dart';
import 'animated_pagination_scroll.dart';
import 'package:flutter/material.dart';

class AnimatedInfiniteScrollView<T extends Object> extends StatefulWidget {
  final PaginationViewModel<T> viewModel;
  final ScrollPhysics? physics;
  final Widget Function(T) itemBuilder;
  final Widget? loadingWidget;
  final Widget? errorWidget;
  final Widget? footerLoadingWidget;
  final bool refreshIndicator;

  const AnimatedInfiniteScrollView({
    required this.viewModel,
    required this.itemBuilder,
    this.footerLoadingWidget,
    this.loadingWidget,
    this.errorWidget,
    this.physics,
    this.refreshIndicator = false,
    Key? key,
  }) : super(key: key);

  @override
  State<AnimatedInfiniteScrollView<T>> createState() => _AnimatedInfiniteScrollViewState<T>();
}

class _AnimatedInfiniteScrollViewState<T extends Object> extends State<AnimatedInfiniteScrollView<T>> {
  final GlobalKey<RefreshIndicatorState> _refreshIndicatorKey = GlobalKey<RefreshIndicatorState>();

  Future<void> _onRefresh() async {
    _refreshIndicatorKey.currentState?.show();
    widget.viewModel.refresh();
    await widget.viewModel.getPaginationList();
    // setState(() {});
    return Future.value();
  }

  Widget _buildScrollView() {
    return AnimatedPaginationScrollView(
      viewModel: widget.viewModel,
      itemBuilder: widget.itemBuilder,
      physics: widget.physics,
      loadingWidget: widget.loadingWidget,
      errorWidget: widget.errorWidget,
      footerLoadingWidget: widget.footerLoadingWidget,
    );
  }

  @override
  Widget build(BuildContext context) {
    return NotificationListener<ScrollNotification>(
      key: UniqueKey(),
      onNotification: (scrollNotification) {
        if (scrollNotification.metrics.pixels == scrollNotification.metrics.maxScrollExtent) {
          widget.viewModel.getPaginationList();
        }
        return true;
      },
      child: widget.refreshIndicator
          ? RefreshIndicator(
              key: _refreshIndicatorKey,
              child: _buildScrollView(),
              onRefresh: _onRefresh,
            )
          : _buildScrollView(),
    );
  }
}
